package de.ruzman.leap;

import java.util.ArrayList;
import java.util.List;

import com.leapmotion.leap.Frame;
import com.leapmotion.leap.Vector;

import de.ruzman.leap.event.PointEvent;
import de.ruzman.leap.event.PointListener;

public abstract class AbstractPoint {	
	private int id;
	protected boolean isActive;

	private Vector diameter;
	private Vector zone;
	protected Vector position;

	private float maxXPos;
	private float minYPos;

	protected TrackingBox trackingBox;
	private List<PointListener> pointListeners;
	private PointEvent pointEvent;

	public AbstractPoint(int id) {
		this(id, new TrackingBox());
	}
	
	public AbstractPoint(int id, TrackingBox trackingBox) {
		this.id = id;

		diameter = new Vector();
		position = new Vector();

		this.trackingBox = trackingBox;
		pointListeners = new ArrayList<>();
		pointEvent = new PointEvent(this);
	}

	public int id() {
		return id;
	}

	protected void update(Frame frame, Vector point) {
		isActive = true;
		
		trackingBox.calcScreenPosition(point, position);
		trackingBox.calcZone(point, zone);

		if (diameter != null) {
			if (position.getX() < 0) {
				position.setX(0);
			} else if (position.getX() > maxXPos) {
				position.setX(maxXPos);
			}

			if (position.getY() < 0) {
				position.setY(0);
			} else if (position.getY() > minYPos) {
				position.setY(minYPos);
			}
		}

		if(pointEvent.update()) {
			for (PointListener pointListener : pointListeners) {
				pointListener.zoneChanged(pointEvent);
			}
		}
	}

	public void addPointListener(PointListener pointListener) {
		pointListeners.add(pointListener);
	}

	public void removePointListener(PointListener pointListener) {
		pointListeners.remove(pointListener);
	}

	public void setActive(boolean isActive) {
		this.isActive = isActive;
	}

	public boolean isActive() {
		return isActive;
	}

	public final void setDiameter(Vector diameter) {
		if (diameter != null) {
			maxXPos = Environment.DISPLAY_WIDTH - diameter.getX();
			minYPos = Environment.DISPLAY_HEIGHT - diameter.getY();
		}

		this.diameter = diameter;
	}

	public boolean hasDiameter() {
		return diameter != null;
	}
	
	public final void setZone(Vector zone) {
		if(this.zone == null) {
			this.zone = zone;
		}
	}
}
