package de.ruzman.leap;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import com.leapmotion.leap.Controller;
import com.leapmotion.leap.Frame;
import com.leapmotion.leap.Hand;
import com.leapmotion.leap.Listener;

import de.ruzman.leap.event.PointListener;

public class MotionRegistry extends Listener {	
	protected ConcurrentMap<Integer, ExtendedHand> hands;
	protected List<PointListener> pointListeners;
	
	public MotionRegistry() {
		hands = new ConcurrentHashMap<>(10);
		pointListeners = new ArrayList<>();
		Environment.getController().addListener(this);
	}
	
	@Override
	public void onDisconnect (Controller controller) {
		clear();
	}
	
	@Override
	public void onExit(Controller controller) {
		clear();
	}
	
	@Override
	public void onFocusLost(Controller controller) {
		clear();
	}
	
	@Override
	public void onFrame(Controller controller) {
		updateHands(controller.frame());
		
	}
	
	protected void updateHands(Frame frame) {
		for(AbstractPoint abstractPoint: hands.values()) {
			abstractPoint.setActive(false);
		}
		
		ExtendedHand extendedHand;
		for(Hand hand: Environment.getFrame().hands()) {
			extendedHand = hands.get(hand.id());

			if(extendedHand == null) {
				extendedHand = new ExtendedHand(hand.id());
				for(PointListener pointListener: pointListeners) {
					extendedHand.addPointListener(pointListener);
				}
				
				hands.put(hand.id(), extendedHand);
			}

			if(extendedHand != null) {
				extendedHand.update(frame, frame.hand(extendedHand.id()).palmPosition());
			}
		}
	}
	
	private void clear() {
		hands.clear();
	}
	
	public void addPointListener(PointListener pointListener) {
		pointListeners.add(pointListener);
		
		for(AbstractPoint abstractPoint: hands.values()) {
			abstractPoint.addPointListener(pointListener);
		}
	}
}
